from unittest.mock import Mock, patch

from django.test import TestCase
from django.urls import reverse

from .models import Page
from .tasks import fetch_all_remote_content

# Create your tests here.


class RemoteContentTests(TestCase):
    def setUp(self):
        self.remote_page = Page.objects.create(
            url="test-remote",
            title="Test Remote",
            kind=Page.REMOTE_CONTENT,
            remote_url="https://example.com/test",
            content="",
        )
        self.regular_page = Page.objects.create(
            url="test-regular",
            title="Test Regular",
            kind=Page.REGULAR,
            content="Regular content",
        )

    @patch("requests.get")
    def test_fetch_remote_content_success(self, mock_get):
        # Mock successful response
        mock_response = Mock()
        mock_response.text = """
        <html>
            <body>
                <h1>Test Content</h1>
                <p>Some test content</p>
            </body>
        </html>
        """
        mock_get.return_value = mock_response

        # Fetch content
        self.remote_page.fetch_remote_content()

        # Verify the page was updated
        self.remote_page.refresh_from_db()
        self.assertIn("<h1>Test Content</h1>", self.remote_page.content)
        self.assertIn("<p>Some test content</p>", self.remote_page.content)
        self.assertFalse(self.remote_page.import_error)
        self.assertTrue(self.remote_page.content.startswith("<div class='content'>"))

    @patch("requests.get")
    def test_fetch_remote_content_error(self, mock_get):
        # Mock failed request
        mock_get.side_effect = Exception("Connection error")

        # Fetch content
        self.remote_page.fetch_remote_content()

        # Verify error state
        self.remote_page.refresh_from_db()
        self.assertTrue(self.remote_page.import_error)
        self.assertEqual(self.remote_page.content, "")

    def test_fetch_regular_page(self):
        # Try to fetch content for a regular page
        self.regular_page.fetch_remote_content()

        # Verify nothing changed
        self.regular_page.refresh_from_db()
        self.assertEqual(self.regular_page.content, "Regular content")
        self.assertFalse(self.regular_page.import_error)

    @patch("requests.get")
    def test_fetch_all_remote_content(self, mock_get):
        # Create another remote page
        Page.objects.create(
            url="test-remote-2",
            title="Test Remote 2",
            kind=Page.REMOTE_CONTENT,
            remote_url="https://example.com/test2",
            content="",
        )

        # Mock successful response
        mock_response = Mock()
        mock_response.text = "<html><body>Test content</body></html>"
        mock_get.return_value = mock_response

        # Run the task
        fetch_all_remote_content()

        # Verify both remote pages were updated
        remote_pages = Page.objects.filter(kind=Page.REMOTE_CONTENT)
        for page in remote_pages:
            self.assertIn("Test content", page.content)
            self.assertFalse(page.import_error)

    def test_page_view_remote_content(self):
        # Test the view with remote content
        url = reverse("pages:view", kwargs={"slug": self.remote_page.url})
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "page.html")
